# core-result-checker
# Copyright (C) 2015 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import argparse
import configparser
import logging
import os

import kombu
from swiftclient.service import SwiftService
from uservice_utils.logging import (
    configure_service_logging,
    ExtraLogger,
)
from uservice_utils.queue import (
    DefaultRetryPolicy,
    MessageActions,
    SimpleRabbitQueueWorker,
)

from core_result_checker import constants


class Worker(object):

    def __init__(self, swift_publisher):
        get_logger(__name__).info("Service Started.")
        self.swift_publisher = swift_publisher

    def __call__(self, message):
        logger = get_logger(__name__, message)
        logger.info("Got: %r", message)
        try:
            channel = message['channel']
            device = message['device']
            image_name = message['image_name']
        except KeyError as e:
            logger.error("Unable to unpack incoming message: %s", str(e))
            return MessageActions.Retry

        container_name = "core-{}-{}-{}".format(
            channel,
            device,
            image_name,
        )
        try:
            self.swift_publisher.make_container_public(container_name)
        except RuntimeError as e:
            logger.error(
                "Unable to publish swift container '%s'. Error: %s.",
                container_name,
                e
            )
            return MessageActions.Retry
        logger.info("Swift container made public OK.")
        return MessageActions.Acknowledge


def get_logger(name, extra={}):
    if logging.getLoggerClass() != ExtraLogger:
        logging.setLoggerClass(ExtraLogger)
    logger = logging.getLogger(name)
    e = constants.LOGGING_EXTRA.copy()
    e.update(extra)
    logger.set_extra_args(e)
    return logger


def read_config():
    parser = argparse.ArgumentParser(description='Core Result Checker.')
    parser.add_argument(
        '-c',
        '--conf',
        default='core-service.conf',
        help='Configuration file path'
    )
    args = parser.parse_args()

    # Load configuration options.
    config = configparser.ConfigParser()
    config.read(args.conf)
    return config


class SwiftPublisher(object):

    """A class that knows how to make a swift container public."""

    def __init__(self, nova_config):
        self.config = nova_config

    def make_container_public(self, container_name):
        """Make a swift contianer public.

        Raises RuntimeError on failure, returns None otherwise.

        """
        swift = SwiftService(self.config)
        result = swift.post(
            container=container_name,
            options=dict(read_acl='.r:*')
        )
        if not result['success']:
            raise RuntimeError(
                "Unable to make swift container '%s' public: %s" % (
                    container_name,
                    result['error']
                )
            )


def main():
    config = read_config()
    log_path = os.path.abspath(
        os.path.join(__file__, '../../logs/core-result-checker.log'))
    configure_service_logging(
        log_path,
        config['logstash'] if 'logstash' in config else None
    )

    amqp_uris = config.get('amqp', 'uris').split()
    swift_publisher = SwiftPublisher(config['nova'])
    worker = Worker(swift_publisher)
    retry_policy = DefaultRetryPolicy(
        max_retries=3,
        dead_queue='core.deadletters.{}'.format(constants.API_VERSION)
    )
    try:
        with kombu.Connection(amqp_uris) as connection:
            queue_monitor = SimpleRabbitQueueWorker(
                connection,
                "core.result.{}".format(constants.API_VERSION),
                worker,
                retry_policy,
            )
            queue_monitor.run()
    except KeyboardInterrupt:
        print("Bye")
